package com.fjhx.utils;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.http.HttpUtil;
import com.fjhx.vo.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.List;
import java.util.function.Consumer;

@Slf4j
@Component
public class EmailEngineUtil {

    public static String urlPrefix;

    public static String attachmentPath;

    private static final RestTemplate restTemplate = new RestTemplate();

    @Value("${config.urlPrefix}")
    public void setUrlPrefix(String urlPrefix) {
        EmailEngineUtil.urlPrefix = urlPrefix;
    }

    @Value("${config.attachmentPath}")
    public void setAttachmentPath(String attachmentPath) {
        EmailEngineUtil.attachmentPath = attachmentPath;
    }

    /**
     * 生成邮箱
     */
    public static void createAccount(BindingVo bindingVo) {
        // 验证邮件配置参数
        VerifyVo verifyVo = createVerifyVo(bindingVo);
        // 验证邮件配置是否正确
        VerifyResult verifyResult = post("v1/verifyAccount", verifyVo, VerifyResult.class);
        // 解析验证结果
        parsingVerifyResult(verifyResult);
        // 生成添加账号入参
        AddAccountVo addAccountVo = createAddAccountVo(verifyVo, bindingVo.getEmail());
        // 添加邮箱
        AddAccountResult addAccountResult = post("v1/account", addAccountVo, AddAccountResult.class);
        // 验证邮箱添加状态
        parsingAddAccountResult(addAccountResult);
    }

    /**
     * 查看邮箱所有文件夹
     */
    public static List<EmailMailboxVo.MailboxesDTO> getMailboxList(String email) {
        String url = "v1/account/" + email + "/mailboxes?counters=false";
        EmailMailboxVo result = get(url, EmailMailboxVo.class);
        return result.getMailboxes();
    }

    /**
     * 查看文件夹的所有邮件
     */
    public static void handleMessageList(String email, String mailboxName, Consumer<List<MessageVo.MessagesDTO>> consumer) {
        int page = 0;
        int pages;

        do {
            String url = "/v1/account/" + email + "/messages?path=" + mailboxName + "&page=" + page + "&pageSize=100&documentStore=false";

            MessageVo result = get(url, MessageVo.class);

            List<MessageVo.MessagesDTO> messages = result.getMessages();
            if (messages.size() > 0) {
                consumer.accept(messages);
            }

            pages = result.getPages();
            page++;
        } while (page < pages);

    }

    /**
     * 获取邮件明细
     */
    public static MessageDetailVo getMessageDetail(String email, String messageId) {
        String url = "v1/account/" + email + "/message/" + messageId + "?textType=html&embedAttachedImages=true&documentStore=false";
        return get(url, MessageDetailVo.class);
    }

    /**
     * 下载附件
     */
    public static void downloadAttachment(String email, String attachmentId, String fileName) throws FileNotFoundException {
        File file = new File(attachmentPath + email);
        if (!file.exists()) {
            boolean mkdir = file.mkdir();
            Assert.isTrue(mkdir, "创建文件夹失败");
        }

        FileOutputStream fileOutputStream = new FileOutputStream(attachmentPath + email + "\\" + fileName);
        download("v1/account/" + email + "/attachment/" + attachmentId, fileOutputStream);
    }

    /**
     * 生成验证邮箱配置实体
     */
    private static VerifyVo createVerifyVo(BindingVo bindingVo) {
        VerifyVo verifyVo = new VerifyVo();

        VerifyVo.Auth auth = new VerifyVo.Auth();
        auth.setUser(bindingVo.getEmail());
        auth.setPass(bindingVo.getSecretKey());

        VerifyVo.Tls tls = new VerifyVo.Tls();
        tls.setRejectUnauthorized(true);
        tls.setMinVersion("TLSv1.2");

        VerifyVo.Imap imap = new VerifyVo.Imap();
        imap.setAuth(auth);
        imap.setPort(bindingVo.getImapPort());
        imap.setHost(bindingVo.getImapHost());
        imap.setTls(tls);
        imap.setSecure(true);
        verifyVo.setImap(imap);

        VerifyVo.Smtp smtp = new VerifyVo.Smtp();
        smtp.setAuth(auth);
        smtp.setPort(bindingVo.getSmtpPort());
        smtp.setHost(bindingVo.getSmtpHost());
        smtp.setTls(tls);
        smtp.setSecure(true);
        verifyVo.setSmtp(smtp);

        return verifyVo;
    }

    /**
     * 解析验证结果
     */
    private static void parsingVerifyResult(VerifyResult verifyResult) {
        VerifyResult.Status imap = verifyResult.getImap();
        VerifyResult.Status smtp = verifyResult.getSmtp();

        Assert.notNull(imap, "imap解析结果为空");
        Assert.notNull(smtp, "smtp解析结果为空");

        Assert.isTrue(imap.getSuccess(), "imap解析失败：error：{} ；code：{}；", imap.getError(), imap.getCode());
        Assert.isTrue(smtp.getSuccess(), "smtp解析失败：error：{} ；code：{}；", smtp.getError(), smtp.getCode());
    }

    /**
     * 生成添加账号入参
     */
    private static AddAccountVo createAddAccountVo(VerifyVo verifyVo, String email) {
        AddAccountVo addAccountVo = BeanUtil.copyProperties(verifyVo, AddAccountVo.class);
        addAccountVo.setAccount(email);
        addAccountVo.setName(email);
        addAccountVo.setEmail(email);
        addAccountVo.setPath("*");
        return addAccountVo;
    }

    /**
     * 验证添加状态
     */
    private static void parsingAddAccountResult(AddAccountResult addAccountResult) {
        String state = addAccountResult.getState();

        Assert.isTrue("new".equals(state) || "existing".equals(state),
                "邮箱添加失败，account：{}，state：{}",
                addAccountResult.getAccount(), addAccountResult.getState());
    }

    private static void download(String url, OutputStream outputStream) {
        HttpUtil.download(urlPrefix + url, outputStream, true);
    }

    public static <T> T get(String url, Class<T> cls) {
        return restTemplate.getForObject(urlPrefix + url, cls);
    }

    public static <T> T post(String url, Object paramObj, Class<T> cls) {
        return restTemplate.postForObject(urlPrefix + url, paramObj, cls);
    }

}
